#region USING
using System;
using System.IO;
using System.Text;
using System.Reflection;
using System.Windows.Forms;
using System.Collections;
using System.Threading;

using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;

using System.Data;

using MSXML2;
using System.Runtime.InteropServices;
#endregion
namespace RSSConnect
{
	/// <summary>
	/// Utility classes and functions
	/// </summary>
	public class Utility
	{
		#region PUBLIC_METHODS
		public void ClearCacheFiles(string extension)
		{
			try
			{
				if(extension == null) return;
				DebugLog.Instance.REPORT("Clearing Cache files " + extension + "...");

				string[] htmlCacheFiles = Directory.GetFiles(Application.StartupPath + @"\cache", extension);
						
				foreach (string cacheFile in htmlCacheFiles)
				{
					try
					{
						File.Delete(cacheFile);
					}
					catch(Exception exp) {DebugLog.Instance.REPORT(exp);};
				}
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
		}

		public DOMDocument30Class DownloadAndSave(string downloadUrl, string localFileName, bool checkInCache)
		{
			//	Works only for XML documents


			//	Using MSXML instead of .NET classes (System.Xml or System.Net)
			//	because of bug in 1.1; See KB: 815209
			//	On Win2K Advanced Server, in some situation, accessing remote XML file
			//	over HTTP produces Socket Exception (hence for example cannot use XmlDocument 
			//	to load remote XML document over HTTP, raises SocketException)
			DOMDocument30Class xmlDoc = null; 

			try
			{
				xmlDoc = new DOMDocument30Class();
				xmlDoc.async = false;
				xmlDoc.validateOnParse = false;
				xmlDoc.setProperty("ServerHTTPRequest", true);
				xmlDoc.resolveExternals = false;

				if(downloadUrl == null || localFileName == null)
					return xmlDoc;

				if(downloadUrl.Length <= 0)
					return xmlDoc;

				string xmlDocToLoad = downloadUrl;
				if(!checkInCache || !File.Exists(localFileName))
				{
					DebugLog.Instance.DEBUG2("Downloading " + downloadUrl + " to " + localFileName);
				}
				else
				{
					xmlDocToLoad = localFileName;
				}

				DebugLog.Instance.DEBUG2("Loading " + xmlDocToLoad + "...");

				bool loadResult = xmlDoc.load(xmlDocToLoad);
				if(!loadResult && (!checkInCache || !File.Exists(localFileName)))
				{	
					DebugLog.Instance.REPORT("Failed while loading XML document " + xmlDocToLoad + " " + xmlDoc.parseError.reason);
					DebugLog.Instance.REPORT("Trying XMLHTTP...");
					
					//	If remote file, still not able to load it,
					//	Try XMLHTTP
					XMLHTTP30Class xmlHttp = new XMLHTTP30Class();
					Object o = null;
					xmlHttp.open("GET", xmlDocToLoad, false, o, o);
					xmlHttp.send(o);

					if(xmlHttp.status == 200)	//	Success
					{
						StreamWriter writer = new StreamWriter(localFileName, false, System.Text.Encoding.Unicode); 
						writer.Write(xmlHttp.responseText);
						writer.Close();
						xmlDocToLoad = localFileName;

						loadResult = xmlDoc.load(xmlDocToLoad);
					}
				}

				if(loadResult)
				{
					if(!checkInCache || !File.Exists(localFileName))
					{
						xmlDoc.save(localFileName);
						DebugLog.Instance.DEBUG2("Successfully loaded and saved to " + xmlDocToLoad);
					}
					else
					{
						DebugLog.Instance.DEBUG2("Successfully loaded " + xmlDocToLoad);
					}
				}
				else
				{
					DebugLog.Instance.REPORT("Failed while loading XML document " + xmlDocToLoad);
					DebugLog.Instance.REPORT(xmlDoc.parseError.reason);
				}
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}

			return xmlDoc;
		}

		public long GetDirSizeInBytes(DirectoryInfo d) 
		{    
			long dirSize = 0;    
			try
			{

				// Add file sizes
				FileInfo[] fis = d.GetFiles();
				foreach (FileInfo fi in fis) 
				{      
					dirSize += fi.Length;    
				}
				
				// Add subdirectory sizes
				DirectoryInfo[] dis = d.GetDirectories();
				foreach (DirectoryInfo di in dis) 
				{
					dirSize += GetDirSizeInBytes(di);   
				}
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
			
			return dirSize;  
		}

		public string FolderSizeToString(long folderSizeInBytes)
		{
			string folderSizeMessage="";
			try
			{
				if (folderSizeInBytes <= 0)
					folderSizeMessage = "0 bytes";
				else if(folderSizeInBytes < 1024)
					folderSizeMessage = folderSizeInBytes.ToString() + " bytes";
				else if(folderSizeInBytes >= 1024 && folderSizeInBytes < 1024*1024)
					folderSizeMessage = Convert.ToInt32(folderSizeInBytes/(1024)).ToString() + " KB";
				else
				{
					Decimal folderMBSize = Convert.ToDecimal(folderSizeInBytes/(1024.00*1024.00));
					folderMBSize = Math.Round(folderMBSize, 2);
					folderSizeMessage = folderMBSize.ToString() + " MB";
				}
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}

			return folderSizeMessage;
		}

		public string GetCacheFileNameFromUrl(string feedUrl)
		{
			string result="";

			try
			{
				if(feedUrl != null)
				{
					result = feedUrl.Replace("http://", "").Replace("/", "").Replace(":", "").Replace("#", "").Replace("?","").Replace(@"\", "").Replace("=", "").Replace("-", "").Replace("_","");
					result = Application.StartupPath + "\\cache\\" + result + ".cache.xml";
				}
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}

			return result;
		}

		public string CheckForAppUpdates()
		{
			string updateMessages = "No new RSSConnect releases are available at this time. Please check back later.";

			//	Using MSXML instead of .NET classes (System.Xml or System.Net)
			//	because of bug in 1.1; See KB: 815209
			DOMDocument30Class updatesXMLDoc = null;
			try
			{
				updatesXMLDoc = new DOMDocument30Class();
				DebugLog.Instance.DEBUG3("Checking for application updates...");
				updatesXMLDoc.async = false;
				updatesXMLDoc.validateOnParse = false;
				updatesXMLDoc.setProperty("ServerHTTPRequest", true);
				updatesXMLDoc.resolveExternals = false;

				string updateUrl = AppOptions.Instance.GetOption("applicationUpdateUrl");
				if(updateUrl == null || (updateUrl != null && updateUrl.Length <= 0))
					updateUrl = "http://www.PerfectXML.com/RSSConnect/RSSConnectUpdates.asp?curVer=";

				//	Append current assembly version
				Assembly assembly = Assembly.GetExecutingAssembly();
				updateUrl += assembly.GetName().Version.ToString();

				DebugLog.Instance.DEBUG3("Loading " + updateUrl);
				
				if(updatesXMLDoc.load(updateUrl))
				{
					IXMLDOMNodeList updateNodes = updatesXMLDoc.selectNodes("/appUpdates/update");
					if(updateNodes.length > 0)
					{
						updateMessages = "";
						int updatesCount = updateNodes.length;
						for(int iIndex=0; iIndex < updatesCount; iIndex++)
						{
							IXMLDOMNode updateNode = updateNodes.nextNode();

							updateMessages += string.Format("Version {0} is now available. {1}\nVisit {2} for more information.\n\n", 
								updateNode.attributes[0].text, 
								updateNode.attributes[2].text,
								updateNode.attributes[1].text);
						}
					}
				}
				else
				{
					DebugLog.Instance.DEBUG3("Load failed! " + updatesXMLDoc.parseError.reason);
				}

			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
			finally 
			{
				try
				{
					if(updatesXMLDoc != null)
						Marshal.ReleaseComObject(updatesXMLDoc);
				}
				catch(Exception) {}
			}

			return updateMessages;
		}

		public Hashtable GetIndexWords(string dataString)
		{
			Hashtable result=new Hashtable(100);	//initial size
			try
			{
				string[] splitWords = dataString.Split(AppOptions.Instance.wordSep);

				//	remove single character words 
				//	remove words starting with a digit (number)
				//	remove stop words
				foreach(string wordString in splitWords)
				{
					//	***Important to Trim
					if(wordString.Trim().Length > 1)
					{
						if(!(wordString[0] >= '0' && wordString[0] <= '9'))
						{
							if(AppOptions.Instance.stopWords[wordString.ToLower()] == null)
							{
								//	No space or tab character allowed
								if(wordString.IndexOf(" ") < 0 && wordString.IndexOf("	") < 0)
								{
									try {result.Add(wordString, wordString);}
									catch(Exception) {}
								}
							}
						}

					}
				}
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}

			return result;
		}

		public bool DataSetToHTMLFile(DataSet sourceDataSet, string outputHTMLFileName)
		{
			bool result = false;
			string xmlFileName = Application.StartupPath + @"\cache\SearchResult.xml";
			try
			{
				//	Save the DataSet XML into a file
				XmlDocument dsXMLDoc = new XmlDocument();
				dsXMLDoc.XmlResolver = null;
				dsXMLDoc.LoadXml(sourceDataSet.GetXml());
				dsXMLDoc.Save(xmlFileName);

				//	Transform DataSet XML into RSS XML
				//	Results go into MemoryStream which is pipelined for next stylesheet
				//	that transforms RSS XML into HTML
				XslTransform dsToRSSTransform = new XslTransform();
				dsToRSSTransform.Load(Application.StartupPath + @"\DataSetXMLToRSS.xslt");
				MemoryStream memStream = new MemoryStream();
				//	To support .NET Framework 1.0, not passing the XmlResolver
				//dsToRSSTransform.Transform(dsXMLDoc, null, memStream, null);
				dsToRSSTransform.Transform(dsXMLDoc, null, memStream);

				//	Save the RSS XML into a file
				memStream.Position = 0;
				dsXMLDoc = new XmlDocument();
				dsXMLDoc.XmlResolver= null;
				dsXMLDoc.Load(memStream);
				dsXMLDoc.Save(xmlFileName);

				//	Apply the current XSLT stylesheet skin to transform RSS XML into HTML
				string xsltStylesheet = AppOptions.Instance.GetOption("xsltFile");
				xsltStylesheet = Application.StartupPath + @"\style\" + xsltStylesheet;
				dsToRSSTransform.Load(xsltStylesheet);

				//	To support .NET Framework 1.0, not passing the XmlResolver
				//dsToRSSTransform.Transform(xmlFileName, result, null);
				dsToRSSTransform.Transform(xmlFileName, outputHTMLFileName);
				result = true;
				memStream.Close();
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}

			return result;
		}

		public int[] ExportFeedToDatabase(string feedUrl)
		{
			int[] result={0,0};
			try
			{
				string feedUrlLocalFileName = GetCacheFileNameFromUrl(feedUrl);
				DownloadAndSave(feedUrl, feedUrlLocalFileName, true);
					
				DBHelperBase dbObject;
				if( (AppOptions.Instance.GetOption("dbType").ToUpper().CompareTo("OLEDB")) == 0 )
					dbObject = new OLEDBHelper(AppOptions.Instance.GetOption("connectionString"));
				else
					dbObject = new SQLMSDEHelper(AppOptions.Instance.GetOption("connectionString"));
			
				if(dbObject.TestConnection())
				{
					result = dbObject.ExportFeedToDB(feedUrlLocalFileName);
				}
				else
				{
					MessageBox.Show("Failed while connecting to the database.", Application.ProductName);
				}

				if((AppOptions.Instance.GetOption("updateIndexAfterExport").CompareTo("1") == 0))
				{
					//	If atleast one record was exported to the database
					if(result[1] > 0)
						RefreshDBIndex();
				}
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}

			return result;
		}

		public int RefreshDBIndex()
		{
			int result = 0;
			try
			{
				DBHelperBase dbObject;
				if( (AppOptions.Instance.GetOption("dbType").ToUpper().CompareTo("OLEDB")) == 0 )
					dbObject = new OLEDBHelper(AppOptions.Instance.GetOption("connectionString"));
				else
					dbObject = new SQLMSDEHelper(AppOptions.Instance.GetOption("connectionString"));

				if(dbObject.TestConnection())
				{
					result = dbObject.IndexDB();
				}
				else
				{
					MessageBox.Show("Failed while connecting to the database.", Application.ProductName);
				}
			
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}

			return result;
		}

		public string PerformSearch(string searchText, searchTypeEnum searchType)
		{
			string result = Application.StartupPath + @"\cache\SearchResult.html";
			try
			{
				DBHelperBase dbObject;
				if( (AppOptions.Instance.GetOption("dbType").ToUpper().CompareTo("OLEDB")) == 0 )
					dbObject = new OLEDBHelper(AppOptions.Instance.GetOption("connectionString"));
				else
					dbObject = new SQLMSDEHelper(AppOptions.Instance.GetOption("connectionString"));
				if(dbObject.TestConnection())
				{
					DataSetToHTMLFile(dbObject.Search(searchText, searchType), result);
				}
				else
				{
					MessageBox.Show("Failed while connecting to the database.", Application.ProductName);
				}
				
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
			return result;
		}

		public string GetValidFileName(string inputFileName)
		{
			string result = inputFileName;
			try
			{
				foreach(char invalidChar in Path.InvalidPathChars)
					result = result.Replace(invalidChar, '_');
				result = result.Replace('/', '_');
				result = result.Replace('\\', '_');
				result = result.Replace('*', '_');
				result = result.Replace('?', '_');
				result = result.Replace(':', '_');
				result = result.Replace('"', '_');
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}

			return result;
		}
		public bool ExportNodeToOPMLFile(TreeNode nodeToExport, string outputFileName)
		{
			bool result = false;
			if(nodeToExport == null || outputFileName == null)
				return result;

			if(outputFileName.Length <= 0)
				return result;

			XmlTextWriter writer = null;

			try
			{
				writer = new XmlTextWriter(outputFileName, Encoding.Unicode);
				writer.Formatting = Formatting.Indented;
				writer.Indentation = 4;

				//	XML Declaration Line
				writer.WriteProcessingInstruction("xml", "version='1.0' encoding='UTF-16'");
				
				//	OPML root node
				writer.WriteStartElement("opml");
				writer.WriteAttributeString("version", "1.0");

				//	head node
				writer.WriteStartElement("head");

				//	TODO: Better handling of root node export (to make sure delete operation works properly)
				string nodeText = nodeToExport.Text;
				if(nodeText.CompareTo("RSSConnect") == 0)
					nodeText = "RSSConnect by PerfectXML.com";
				writer.WriteElementString("title", nodeText);
				writer.WriteElementString("dateCreated", 
					System.DateTime.Now.ToUniversalTime().ToString("r"));
				writer.WriteEndElement();

				//	body node
				writer.WriteStartElement("body");

				//	outline nodes (recursive method)
				BuildOPMLOutlineNodes(writer, nodeToExport);

				writer.WriteEndElement();//	body
				writer.WriteEndElement();//	opml
				
				result = true;
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
			finally 
			{
				if(writer != null)
					writer.Close();
			}

			return result;
		}

		private void BuildOPMLOutlineNodes(XmlWriter writer, TreeNode nodeToExport)
		{
			if(nodeToExport == null || writer == null)
				return;

			try
			{
				string nodeText = nodeToExport.Text;
				
				//	TODO: Better handling of root node export (to make sure delete operation works properly)
				if(nodeText.CompareTo("RSSConnect") == 0)
					nodeText = "RSSConnect by PerfectXML.com";

				writer.WriteStartElement("outline");
				writer.WriteAttributeString("text", nodeText);
				writer.WriteAttributeString("title", nodeText);
				if(nodeToExport.Tag != null)
				{
					if(((TreeNodeTag)(nodeToExport.Tag)).feedUrl != null)
					{
						writer.WriteAttributeString("type", "rss");
						writer.WriteAttributeString("xmlUrl", ((TreeNodeTag)(nodeToExport.Tag)).feedUrl);
					}
				}
				foreach(TreeNode curNode in nodeToExport.Nodes)
				{
					if(curNode != null)
					{
						writer.Flush();
						BuildOPMLOutlineNodes(writer, curNode);
					}
				}

				writer.WriteEndElement();
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
		}
		#endregion
	}
	#region OTHER_DECLARATIONS
	[Flags()]
	public enum nodeTypeEnum 
	{ 
		feedNode=1, 
		folderNode=2, 
		opmlNode=4, 
		childNodeToOPML=8
	};
	public enum feedTreeImageEnum
	{
		folder=0,
		home=1,
		feed=2,
		opml=3
	};
	class TreeNodeTag
	{
		public string feedUrl;
		public string opmlUrl;
		public nodeTypeEnum nodeType;

		public TreeNodeTag()
		{
			feedUrl = null;
			opmlUrl = null;
			nodeType = nodeTypeEnum.feedNode;
		}

	}


	//	When rapid up/down arrow keys are pressed on the tree, we don't want to handle each 
	//	node select event. On select event, start a timer thread after one second, and if during
	//	this, another select event is recd, skip it.
	//	Following class is used for thread synchronization
	class Synchronizer
	{
		public bool IsInThread;
		public Synchronizer()
		{
			IsInThread = false;
		}
	}

	public enum DBType 
	{
		OLEDB,
		SQLorMSDE
	}

	public class NewFeedInfo
	{
		public string feedUrl;
		public string feedTitle;
		public string parentFolderName;
	}

	public enum searchTypeEnum
	{
		anyWords,
		allWords,
		exactPhrase
	}

	#endregion
}
